"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityJwtAuthGuard = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const core_1 = require("@nestjs/core");
const jwt_1 = require("@nestjs/jwt");
const microservices_exceptions_1 = require("@devseeder/microservices-exceptions");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const custom_jwt_auth_guard_1 = require("./custom-jwt-auth.guard");
const error_auth_keys_enum_1 = require("../../enum/error-auth-keys.enum");
let EntityJwtAuthGuard = class EntityJwtAuthGuard extends custom_jwt_auth_guard_1.CustomJwtAuthGuard {
    constructor(reflector, jwtService, configService, scopeKey, entitySchemaData, errorService) {
        super(reflector, jwtService, configService, `${scopeKey}/ADM`);
        this.reflector = reflector;
        this.jwtService = jwtService;
        this.configService = configService;
        this.scopeKey = scopeKey;
        this.entitySchemaData = entitySchemaData;
        this.errorService = errorService;
    }
    canActivate(context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const bearerToken = this.getAuthToken(context, 'Bearer').replace('Bearer ', '');
                const tokenPayload = yield this.jwtService.verifyAsync(bearerToken, {
                    secret: this.configService.get('auth.jwt.secret'),
                    ignoreExpiration: true,
                });
                const metaScope = this.reflector.get('metaScopes', context.getHandler());
                if (!metaScope || !Object.keys(metaScope).length)
                    return true;
                if (tokenPayload.scopes.indexOf(this.scopeADM) !== -1)
                    return true;
                const scopes = this.getScopes(metaScope);
                let scopesPermission = 0;
                scopes.forEach((scope) => {
                    if (tokenPayload.scopes.indexOf(scope) > -1)
                        scopesPermission++;
                });
                if (!scopesPermission)
                    yield this.errorService.throwError(error_auth_keys_enum_1.ErrorAuthKey.MISSING_SCOPE_AUTH);
                return true;
            }
            catch (err) {
                if (err instanceof microservices_exceptions_1.CustomErrorException)
                    throw err;
                throw new microservices_exceptions_1.CustomErrorException(err.message, common_1.HttpStatus.UNAUTHORIZED);
            }
        });
    }
    getScopes(metaScopeInfo) {
        const entitySchema = this.entitySchemaData.filter((ent) => metaScopeInfo.entity === ent.entity);
        if (!entitySchema.length)
            return [];
        const requestScopes = entitySchema[0].authScopes.filter((req) => req.accessKey === metaScopeInfo.accessKey);
        if (!requestScopes.length)
            return [];
        const schemaScopes = requestScopes[0].scopes.map((scope) => `${this.scopeKey}/${scope.key}`);
        return [
            ...schemaScopes,
            `${this.scopeKey}/${metaScopeInfo.entity.toUpperCase()}/${metaScopeInfo.accessKey.toUpperCase()}`,
        ];
    }
};
exports.EntityJwtAuthGuard = EntityJwtAuthGuard;
exports.EntityJwtAuthGuard = EntityJwtAuthGuard = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [core_1.Reflector,
        jwt_1.JwtService,
        config_1.ConfigService, String, Array, nestjs_microservices_schemas_1.ErrorService])
], EntityJwtAuthGuard);
